package yrpg.application.crmvisit.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.ExtendedDataEntity;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.AddValidatorsEventArgs;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.EndOperationTransactionArgs;
import kd.bos.entity.validate.AbstractValidator;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;

import java.util.Date;
import java.util.List;

/**
 * 单据操作插件NewVisitOpPlugIn
 * 新建拜访
 */
public class NewVisitOpPlugin extends AbstractOperationServicePlugIn implements Plugin {

    @Override
    public void onPreparePropertys(PreparePropertysEventArgs e) {
        super.onPreparePropertys(e);
        List<String> fieldKeys = e.getFieldKeys();
        fieldKeys.add("yrpg_taskstatus");
        fieldKeys.add("yrpg_sign_in_time");
        fieldKeys.add("yrpg_sign_out_time");
        fieldKeys.add("yrpg_textareafield1");
        fieldKeys.add("yrpg_sign_in_img_1");
        fieldKeys.add("yrpg_sign_in_img_2");
        fieldKeys.add("yrpg_sign_out_img_1");
        fieldKeys.add("yrpg_sign_out_img_2");
    }

    @Override
    public void onAddValidators(AddValidatorsEventArgs e) {
        super.onAddValidators(e);
        e.addValidator(new NewVisitDateValidator());


    }
    class NewVisitDateValidator extends AbstractValidator {

        @Override
        public void validate() {
            String opKey = this.getOperateKey();
            ExtendedDataEntity[] dataEntities = getDataEntities();
            for (ExtendedDataEntity data : dataEntities) {
                DynamicObject entity = data.getDataEntity();
                if (opKey.equals("signin")||opKey.equals("sign_in_sp")) {
                    //签到
                    String img_1 = entity.getString("yrpg_sign_in_img_1");
                    String img_2 = entity.getString("yrpg_sign_in_img_2");
                    if (img_1.isEmpty() && img_2.isEmpty()) {
                        this.addMessage(data,"请上传签到图片");
                    }
                } else if (opKey.equals("signout")||opKey.equals("sign_out_sp")) {
                    //签退
                    String img_1 = entity.getString("yrpg_sign_out_img_1");
                    String img_2 = entity.getString("yrpg_sign_out_img_2");
                    if (img_1.isEmpty() && img_2.isEmpty()) {
                        this.addMessage(data,"请上传签到图片");
                    }
                    Date in_time = entity.getDate("yrpg_sign_in_time");
                    Date out_time =new Date();
                    if (in_time != null && out_time != null) {
                        long minutes = (out_time.getTime() - in_time.getTime()) / (60 * 1000);
                        if (minutes < 10) {
                            this.addMessage(data,"还未到签退时间");
                        }
                    } else {
                        this.addMessage(data,"请重新定位");
                    }
                } else if (opKey.equals("summary")) {
                    //总结
                    String summarize = entity.getString("yrpg_textareafield1").trim();
                    if (summarize.isEmpty()) {
                        this.addMessage(data,"请填写总结");
                    }
                }
            }
        }
    }

    @Override
    public void endOperationTransaction(EndOperationTransactionArgs e) {
        super.endOperationTransaction(e);
        String opKey = e.getOperationKey();
        for (DynamicObject data : e.getDataEntities()) {
            if (opKey.equals("submit")) {
                //提交
                data.set("yrpg_taskstatus","B");

            }else if (opKey.equals("signin")) {
                //签到
                data.set("yrpg_taskstatus","C");
                data.set("yrpg_sign_in_time", new Date());
            }else if (opKey.equals("signout")) {
                //签退
                data.set("yrpg_taskstatus","D");
                data.set("yrpg_sign_out_time", new Date());
            }else if (opKey.equals("summary")) {
                //总结
                String summarize = data.getString("yrpg_textareafield1").trim();
                if (!summarize.isEmpty()) {
                    data.set("yrpg_taskstatus", "E");
                }
            }
            if (opKey.equals("submit")) {
                SaveServiceHelper.update(data);
            }else {
                SaveServiceHelper.save(new DynamicObject[]{data});
            }
        }
    }
}